# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


class PostFinanceCheckoutSdkException(Exception):
    """
    Exception thrown when an internal SDK error occurs
    """
    def __init__(self, code: str, message: str):
        """
        Constructor.

        :param code: SDK error code (string)
        :param message: Exception message details
        """
        self.code = code
        self.message = message
        super().__init__(f"Error code: {code}. {message}")

    def __str__(self):
        return super().__str__()