# coding: utf-8

"""
PostFinance Python SDK

This library allows to interact with the PostFinance payment service.

Copyright owner: Wallee AG
Website: https://www.postfinance.ch/en/private.html
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from postfinancecheckout.models.payment_terminal_configuration_version import PaymentTerminalConfigurationVersion
from postfinancecheckout.models.payment_terminal_location_version import PaymentTerminalLocationVersion
from postfinancecheckout.models.payment_terminal_state import PaymentTerminalState
from postfinancecheckout.models.payment_terminal_type import PaymentTerminalType
from typing import Optional, Set
from typing_extensions import Self

class PaymentTerminal(BaseModel):
    """
    PaymentTerminal
    """ # noqa: E501
    identifier: Optional[StrictStr] = Field(default=None, description="The unique identifier of the terminal, that is displayed on the device.")
    planned_purge_date: Optional[datetime] = Field(default=None, description="The date and time when the object is planned to be permanently removed. If the value is empty, the object will not be removed.", alias="plannedPurgeDate")
    external_id: Optional[StrictStr] = Field(default=None, description="A client-generated nonce which uniquely identifies some action to be executed. Subsequent requests with the same external ID do not execute the action again, but return the original result.", alias="externalId")
    type: Optional[PaymentTerminalType] = None
    device_name: Optional[StrictStr] = Field(default=None, description="The name of the device that is currently linked to the payment terminal.", alias="deviceName")
    version: Optional[StrictInt] = Field(default=None, description="The version is used for optimistic locking and incremented whenever the object is updated.")
    device_serial_number: Optional[StrictStr] = Field(default=None, description="The serial number of the device that is currently linked to the payment terminal.", alias="deviceSerialNumber")
    linked_space_id: Optional[StrictInt] = Field(default=None, description="The ID of the space this object belongs to.", alias="linkedSpaceId")
    configuration_version: Optional[PaymentTerminalConfigurationVersion] = Field(default=None, alias="configurationVersion")
    location_version: Optional[PaymentTerminalLocationVersion] = Field(default=None, alias="locationVersion")
    default_currency: Optional[StrictStr] = Field(default=None, description="The default currency of the terminal.", alias="defaultCurrency")
    name: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="The name used to identify the payment terminal.")
    id: Optional[StrictInt] = Field(default=None, description="A unique identifier for the object.")
    state: Optional[PaymentTerminalState] = None
    __properties: ClassVar[List[str]] = ["identifier", "plannedPurgeDate", "externalId", "type", "deviceName", "version", "deviceSerialNumber", "linkedSpaceId", "configurationVersion", "locationVersion", "defaultCurrency", "name", "id", "state"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of PaymentTerminal from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "identifier",
            "planned_purge_date",
            "external_id",
            "device_name",
            "version",
            "device_serial_number",
            "linked_space_id",
            "default_currency",
            "name",
            "id",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of type
        if self.type:
            _dict['type'] = self.type.to_dict()
        # override the default output from pydantic by calling `to_dict()` of configuration_version
        if self.configuration_version:
            _dict['configurationVersion'] = self.configuration_version.to_dict()
        # override the default output from pydantic by calling `to_dict()` of location_version
        if self.location_version:
            _dict['locationVersion'] = self.location_version.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of PaymentTerminal from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "identifier": obj.get("identifier"),
            "plannedPurgeDate": obj.get("plannedPurgeDate"),
            "externalId": obj.get("externalId"),
            "type": PaymentTerminalType.from_dict(obj["type"]) if obj.get("type") is not None else None,
            "deviceName": obj.get("deviceName"),
            "version": obj.get("version"),
            "deviceSerialNumber": obj.get("deviceSerialNumber"),
            "linkedSpaceId": obj.get("linkedSpaceId"),
            "configurationVersion": PaymentTerminalConfigurationVersion.from_dict(obj["configurationVersion"]) if obj.get("configurationVersion") is not None else None,
            "locationVersion": PaymentTerminalLocationVersion.from_dict(obj["locationVersion"]) if obj.get("locationVersion") is not None else None,
            "defaultCurrency": obj.get("defaultCurrency"),
            "name": obj.get("name"),
            "id": obj.get("id"),
            "state": obj.get("state")
        })
        return _obj


